////////////////////////////////////////////////////////////////////////////////
// Project:   Event Capture
// Patch:     EC*2*114
// Comments:
//            Changes to the source code related to patch EC*2*114 are supplied
//            with comments referring ICD-10.
//
//            Conditionals:
//              AANTEST - code included for time of testing
//                        (disabling some validations, GUI appearance etc.)
//
////////////////////////////////////////////////////////////////////////////////

unit UguiUtilities;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DBGrids, Menus, ExtCtrls, ComCtrls, ToolWin, Db, DBTables, StdCtrls,
  Buttons, DBCtrls, Trpcb, MFunStr, ORFn, ORNet, ORCtrls, DiTypLib;

const
  IB_BRKERR = 'Broker Error';
  IB_CONFIRM = 'Confirmation';
  IB_DATAERR = 'Data Error';
  IB_EC = 'Event Capture';
  IB_FHELP = 'Field Help';
  IB_INFO = 'Information';
  IB_INVKEY = 'Invalid Key';
  IB_LISTERR = 'Error Creating List';
  IB_NOREC = 'No Records Found';
  IB_NOTADDED = 'Record Not Added';
  IB_NOTFILED = 'Record Not Filed';
  IB_REQDATA = 'Required Data Missing';
  IB_SELREQ = 'Selection Required';
  IB_WARN = 'Warning';

type
  TInterfaceObject = class(TObject)
  //     PtIEN     ------>  Patient IEN
  //     PdIEN     ------>  Provider IEN
  //     vstIEN    ------>  Visit IEN ( ptr to #9000010 )
  //     locIEN    ------>  Location IEN
  //     Date      ------>  Date/Time
  //     DUZ       ------>  User DUZ

  private
    procedure SetPtIEN(newValue: String);
    procedure SetPdIEN(newValue: String);
    procedure SetVstIEN(newValue: String);
    procedure SetLocIEN(newValue: String);
    procedure SetDateStr(newValue: String);
    procedure SetDUZ(newValue: String);
    procedure SetInterfaceUsed(newValue: Boolean);
  protected
    FsPtIEN: String;
    FsPdIEN: String;
    FsVstIEN: String;
    FsLocIEN: String;
    FsDateStr: String;
    FsDUZ: String;
    FbInterfaceUsed: Boolean;
  public
    { Public Declarations }
    constructor Create;
    destructor  Destroy; override;
    property PtIEN: String read FsPtIEN write SetPtIEN;
    property PdIEN: String read FsPdIEN write SetPdIEN;
    property VstIEN: String read FsVstIEN write SetVstIEN;
    property LocIEN: String read FsLocIEN write SetLocIEN;
    property DateStr: String read FsDateStr write SetDateStr;
    property DUZ: String read FsDUZ write SetDUZ;
    property InterfaceUsed: Boolean read FbInterfaceUsed write SetInterfaceUsed;
  end;


type
  TIncSearch = class(TObject)

  private
    procedure SetSearchString(newValue: String);
    procedure SetIndexInteger(newValue: Integer);
  protected
    FSearchString: String;
    FIndexInteger: Integer;
  public
    { Public Declarations }
    constructor Create;
    destructor  Destroy; override;
    property SearchString: String read FSearchString write SetSearchString;
    property IndexInteger: Integer read FIndexInteger write SetIndexInteger;
//  PIncSearch = ^IncSearch;
//  IncSearch = record
//     SearchString: String;
//     IndexInteger: Integer;
  end;

type
  PListItemRow = ^ListItemRow;
  ListItemRow = record
     RecordIEN: String;
  end;

type
  TUserLastUsed = class(TObject)    // ec*100 SRS 3.2.13, 3.2.14

  private
  protected
    FClinic: String;
    FLocation: String;
    FPrinter: String;
    FecHandle: String;
    FPatient: String;
    FVisit: String;
  public
    { Public Declarations }
    constructor Create;
    destructor  Destroy; override;
    property Clinic: String read FClinic write FClinic;
    property Location: String read FLocation write FLocation;
    property Printer: String read FPrinter write FPrinter;
    property ecHandle: String read FecHandle write FecHandle;
    property Patient: String read FPatient write FPatient;
    property Visit: String read FVisit write FVisit;
    procedure Load;
    procedure Save;
   end;

type
  TempLastUsed = record
      LastUsedClinic: string;
      LastUsedLocation: string;
      LastUsedPrinter: string;
    end;

  function  SetupIncrementalSearch(var KeyParam: Char; IncSearchParam: TIncSearch): TIncSearch;
  //added function below for 1_6 3.2.12 to allow - and / in the search - cvw
  function  SetupMgmtDSSUIncrementalSearch(var KeyParam: Char; IncSearchParam: TIncSearch): TIncSearch;
  //added function below:
  function  SetUpDssAndLocationSearch(var KeyParam: Char; IncSearchParam: TIncSearch): TIncSearch;
  procedure PerformIncrementalSearch(IncSearchParam: TIncSearch; ComboBoxParam: TComboBox);
  procedure PerformGridTitleSort(MyGrid: TDBGrid; MyField: TField; AscDesc: String);
  procedure GetEcFieldHelp(FileParam, FieldParam: String);
  procedure GetEcScreenHelp(IenParam: String; MaxLinesParam: Integer);
  procedure DisplayTextWithFieldHelp(TextParam, FileParam, FieldParam: String);
  procedure SetGroupBoxColor(gboxParam: TGroupBox);
  procedure SetComboBoxColor(cboxParam: TComboBox);
  function  SetupPopupMenuItems(MyGrid: TDBGrid; MyPopupMenu: TPopupMenu; MyColumn: TColumn): TField;
  function  CallServer(RPC: String; CallingParams: Array of String; MultList: TStrings): TStrings;
  function  FormatDateTimeForDisplay(DateParam: TDate; TimeParam, DisplayParam: String): String;
  function  SetupDate(TextParam: String): TDate;
  //acs - added function GetDateTime:
  function  GetDateTime(TextParam: String): String;
  function  SetMonthValue(MonthParam: String): String;
  function  ConvertTwoDigitYear(YearParam: Integer): String;
  function  DateToMDate(DateParam: String): String;

  function WindowsDateToFMDate(WinDate: TDate): Double; // zzzzzzandria 2012-04-16

  function  SetMdateOfProcedure(TextParam: String): String;
  function  SubSetOfRecords(const SearchParams: String): TStrings;
  function  GetGenericIEN(IndexParam, ValueParam, FileNoParam, FileNameParam: String): String;
  function  CompareVersion(const A, B: String): Integer;
  function  InfoBox(const Text, Caption: string; Flags: Word): Integer;
  function  ScreenReaderActive: Boolean;
  procedure ShowRPCErrors(ErrorCode: String; Prompt: String; StartPiece: Integer);  // rpk 6/26/2008
  procedure CaptionListbox508msg(HdrCntl: Theadercontrol; kind: integer ) ; //ECS FY12 CVW
  function IsStrANumber(const S: string): Boolean ; //ECS FY13 CVW
  procedure PutControlInGroupBox(TargetControl: TWinControl; GBCaption: string; ParentForm: TForm) ; //ECS FY14 ;
type
  TECSTimeoutCondition = function: Boolean of object;
  TECSTimeoutAction = procedure of object;

procedure InitTimeOut(AUserCondition: TECSTimeoutCondition;
                      AUserAction: TECSTimeoutAction);
procedure UpdateTimeOutInterval(NewTime: Cardinal);
procedure ShutDownTimeOut;
function  TimedOut: Boolean;
function SPiece(x: string; del: string; piece1: integer = 1; piece2: integer=0) : string;

implementation

uses ShellAPI, UfrmECMain, UfrmDisplayFormHelp, UfrmEcTimeout
{$IFDEF AANTEST}
  , uZZ_RPCEvent, fZZ_EventLog   // zzzzzzandria 2012-04-16 ====================
{$ENDIF}
  ;

type
  TECSTimeoutTimer = class(TTimer)

  private
    FHooked: Boolean;
    FUserCondition: TECSTimeoutCondition;
    FUserAction: TECSTimeoutAction;
    uTimeoutInterval: Cardinal;
    uTimeoutKeyHandle, uTimeoutMouseHandle: HHOOK;
  protected
    procedure ResetTimeout;
    procedure EcsTimeout(Sender: TObject);
  end;

var
  tmrTimeout: TECSTimeoutTimer = nil;
  FTimedOut: Boolean = FALSE;

function TimeoutKeyHook(Code: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; StdCall; forward;
function TimeoutMouseHook(Code: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; StdCall; forward;

constructor TInterfaceObject.Create;
begin
  inherited;
end;

destructor TInterfaceObject.Destroy;
begin
  inherited;
end;

// copied from MFunStr:Piece;
// Piece is also found in ORFn with a char Delim arg
function SPiece(x: string; del: string; piece1: integer = 1; piece2: integer=0) : string;
{PIECE(string,delimiter,piece number)
Returns a field within a string using the specified delimiter.}
var
  delIndex,pieceNumber: integer;
  Resval: String;
  Str: String;
begin
  {initialize variables}
  pieceNumber := 1;
  Str := x;
  {delIndex :=1;}
  if piece2 = 0 then
    piece2 := piece1;
  Resval := '';
  repeat
    delIndex := Pos(del,Str);
    if (delIndex > 0) or ((pieceNumber > Pred(piece1)) and (pieceNumber < (piece2+1))) then begin
      if (pieceNumber > Pred(piece1)) and (pieceNumber < (piece2+1)) then
      begin
        if (pieceNumber > piece1) and (Str <> '') then
          Resval := Resval + del;
        if delIndex > 0 then
        begin
          Resval := Resval + Copy(Str,1,delIndex-1);
          Str := Copy(Str,delIndex+Length(del),Length(Str));
        end
        else
        begin
          Resval := Resval + Str;
          Str := '';
        end;
      end
      else
        Str := Copy(Str,delIndex+Length(del),Length(Str));
    end
    else if Str <> '' then
      Str := '';
    inc(pieceNumber);
  until (pieceNumber > piece2);
  Result := Resval;
end;

procedure TInterfaceObject.SetPtIEN(newValue: String);
begin
  FsPtIEN := newValue;
end;

procedure TInterfaceObject.SetPdIEN(newValue: String);
begin
  FsPdIEN := newValue;
end;

procedure TInterfaceObject.SetVstIEN(newValue: String);
begin
  FsVstIEN := newValue;
end;

procedure TInterfaceObject.SetLocIEN(newValue: String);
begin
  FsLocIEN := newValue;
end;

procedure TInterfaceObject.SetDateStr(newValue: String);
begin
  FsDateStr := newValue;
end;

procedure TInterfaceObject.SetDUZ(newValue: String);
begin
  FsDUZ := newValue;
end;

procedure TInterfaceObject.SetInterfaceUsed(newValue: Boolean);
begin
  FbInterfaceUsed := newValue;
end;

constructor TIncSearch.Create;
begin
  inherited;
end;

destructor TIncSearch.Destroy;
begin
  inherited;
end;

procedure TIncSearch.SetSearchString(newValue: String);
begin
  FSearchString := newValue;
end;

procedure TIncSearch.SetIndexInteger(newValue: Integer);
begin
  FIndexInteger := newValue;
end;

constructor TUserLastUsed.Create;
begin
  inherited;
end;

destructor TUserLastUsed.Destroy;
begin
  inherited;
end;

procedure TUserLastUsed.Load;
var
  rpcParam: string;
  ResultList: TStringList;
begin
  try
    ResultList := TStringList.Create;
    rpcParam := 'Constructor.EC OBU LAST USED';
    ResultList.Assign(CallServer('ECOB FACTORY',rpcParam,nil));
    ecHandle := ResultList[0];

    rpcParam := ecHandle + '.GetLocation';
    ResultList.Assign(CallServer('ECOB METHOD',rpcParam,nil));
    Location := ResultList[0];

    rpcParam := ecHandle + '.GetClinic';
    ResultList.Assign(CallServer('ECOB METHOD',rpcParam,nil));
    Clinic := ResultList[0];

    rpcParam := ecHandle + '.GetPrinter';
    ResultList.Assign(CallServer('ECOB METHOD',rpcParam,nil));
    Printer := ResultList[0];

    except
         On EBrokerError do
           InfoBox('Broker Error at Load Parameters.', IB_BRKERR, mb_OK);
    end;
end;

procedure TUserLastUsed.Save;
var
  rpcParam: string;
  ResultList: TStringList;
begin
  ResultList := TStringList.Create;
  try
    rpcParam := ecHandle + '.SetLocation.' + Location;
    ResultList.Assign(CallServer('ECOB METHOD',rpcParam,nil));

    rpcParam := ecHandle + '.SetClinic.' + Clinic;
    ResultList.Assign(CallServer('ECOB METHOD',rpcParam,nil));

    rpcParam := ecHandle + '.SetPrinter.' + Printer;
    ResultList.Assign(CallServer('ECOB METHOD',rpcParam,nil));

    rpcParam := ecHandle + '.Save';
    ResultList.Assign(CallServer('ECOB METHOD',rpcParam,nil));

    rpcParam := 'Destructor.' + ecHandle;
    ResultList.Assign(CallServer('ECOB FACTORY',rpcParam,nil));

  except
    On EBrokerError do
        InfoBox('Broker Error at Save Parameters.', IB_BRKERR, mb_OK);
  end;
end;

//acs - added this function to handle the incremental search on the DSS Unit and
//Location field which should allow all characters
function SetupDssAndLocationSearch(var KeyParam: Char; IncSearchParam: TIncSearch): TIncSearch;
var
  isTemp: TIncSearch;
begin
  isTemp := IncSearchParam;
  if Ord(KeyParam) = VK_BACK then
     if Length(isTemp.SearchString) = 0 then
        begin
          KeyParam := #0;
          Beep;
        end
     else
        isTemp.SearchString := Copy(isTemp.SearchString, 1, Length(isTemp.SearchString) - 1)
  else
     isTemp.SearchString := isTemp.SearchString + KeyParam;
  Result := isTemp;
end;


function SetupIncrementalSearch(var KeyParam: Char; IncSearchParam: TIncSearch): TIncSearch;
var
  isTemp: TIncSearch;
begin
  isTemp := IncSearchParam;
  if KeyParam in ['0'..'9', ' ', '<', 'A'..'Z'] then
     isTemp.SearchString := isTemp.SearchString + KeyParam
  // start modified code
  // comment out modified code
  //   else if (Ord(KeyParam) = VK_RETURN) and (isTemp.SearchString = ' ') then begin end
  // end modified code
  else if Ord(KeyParam) = VK_BACK then
     begin
       if Length(isTemp.SearchString) = 0 then
          begin
            KeyParam := #0;
            Beep;
          end
       else
          isTemp.SearchString := Copy(isTemp.SearchString, 1, Length(isTemp.SearchString) - 1);
     end
  else
     // ShowMessage replaced by InfoBox - Patch 84 - tjl
     InfoBox('Invalid key pressed.  Please try again.', IB_INVKEY, mb_OK);
  Result := isTemp;
end;

  //added function below for 1_6 3.2.12 to allow - and / in the search - cvw
function SetupMgmtDSSUIncrementalSearch(var KeyParam: Char; IncSearchParam: TIncSearch): TIncSearch;
var
  isTemp: TIncSearch;
begin
  isTemp := IncSearchParam;
  if KeyParam in ['0'..'9', ' ', '<', '-', '/', 'A'..'Z'] then
     isTemp.SearchString := isTemp.SearchString + KeyParam
  // start modified code
  // comment out modified code
  //   else if (Ord(KeyParam) = VK_RETURN) and (isTemp.SearchString = ' ') then begin end
  // end modified code
  else if Ord(KeyParam) = VK_BACK then
     begin
       if Length(isTemp.SearchString) = 0 then
          begin
            KeyParam := #0;
            Beep;
          end
       else
          isTemp.SearchString := Copy(isTemp.SearchString, 1, Length(isTemp.SearchString) - 1);
     end
  else
     // ShowMessage replaced by InfoBox - Patch 84 - tjl
     InfoBox('Invalid key pressed.  Please try again.', IB_INVKEY, mb_OK);
  Result := isTemp;
end;

procedure PerformIncrementalSearch(IncSearchParam: TIncSearch; ComboBoxParam: TComboBox);
var
  isTemp: TIncSearch;
  cboxTemp: TComboBox;
  TheLength, TheIndex: Integer;
  ThePChar: array[0..30] of char;
begin
  isTemp := IncSearchParam;
  cboxTemp := ComboBoxParam;
  // start modified code
  if Length(isTemp.SearchString) > 0 then
  // comment out modified code
  //    if (Length(isTemp.SearchString) > 0) and (isTemp.SearchString <> ' ') then
  // end modified code
     begin
        if Length(isTemp.SearchString) > 29 then TheLength := 30
        else TheLength := Length(isTemp.SearchString);
        StrPCopy(ThePChar, Copy(isTemp.SearchString, 1, TheLength));
        TheIndex := SendMessage(cboxTemp.Handle, cb_SelectString, -1, LongInt(@ThePChar));
        if TheIndex = -1 then
             isTemp.SearchString := Copy(isTemp.SearchString, 1, TheLength - 1)
        else isTemp.IndexInteger := TheIndex;
     end;
end;

procedure GetEcFieldHelp(FileParam, FieldParam: String);
begin
  //rpc 'ddr get dd help' will do the same
{
procedure DisplayHelpInMemo(HelpMemo:TMemo; HelpCmp:
  TFMHelp; Fld, Fn, Flgs: string);
var
  MyHelp: TFMHelpObj;
       i: integer;
begin
  HelpCmp.FieldNumber:=Fld;
  HelpCmp.FileNumber:=Fn;
  HelpCmp.HelpFlags:=Flgs;
  MyHelp:=HelpCmp.GetHelp;
  HelpMemo.Clear;
  for i:=0 to (MyHelp.HelpText.Count-1) do
    HelpMemo.Lines.Add(MyHelp.HelpText[i]);
end;
}
  frmECMain.fmHelpGeneric.FileNumber  := FileParam;
  frmECMain.fmHelpGeneric.FieldNumber := FieldParam;
  frmECMain.fmHelpGeneric.HelpFlags   := 'HD';
  frmECMain.fmHelpGeneric.GetAndDisplayHelp;
end;

procedure DisplayTextWithFieldHelp(TextParam, FileParam, FieldParam: String);
var
  fmHelpObject: TFMHelpObj;
begin
  frmECMain.fmHelpGeneric.FileNumber  := FileParam;
  frmECMain.fmHelpGeneric.FieldNumber := FieldParam;
  frmECMain.fmHelpGeneric.HelpFlags   := 'HD';
  fmHelpObject := frmECMain.fmHelpGeneric.GetHelp;
  if Assigned(fmHelpObject) then
     begin
       fmHelpObject.HelpText.Insert(0, TextParam);
       fmHelpObject.DisplayHelp;
     end;
end;

procedure GetEcScreenHelp(IenParam: String; MaxLinesParam: Integer);
begin
  frmDisplayFormHelp := TfrmDisplayFormHelp.CreateUpdate(IenParam, MaxLinesParam, Application);
  frmDisplayFormHelp.ShowModal;
  frmDisplayFormHelp.Free;
  frmDisplayFormHelp := nil;
end;

procedure SetGroupBoxColor(gboxParam: TGroupBox);
var
  i: Integer;
  ChildControl: TControl;
begin
  for i := 0 to (gboxParam.ControlCount - 1) do
    begin
      ChildControl := gboxParam.Controls[i];

      if (ChildControl.ClassName = 'TListBox') then
         if gboxParam.Enabled then
              (ChildControl as TListBox).Color := clWindow
         else (ChildControl as TListBox).Color := clBtnFace;

      if (ChildControl.ClassName = 'TComboBox') then
         begin
           (ChildControl as TComboBox).Enabled := gboxParam.Enabled;
           if gboxParam.Enabled then
                (ChildControl as TComboBox).Color := clWindow
           else (ChildControl as TComboBox).Color := clBtnFace;
         end;

      if (ChildControl.ClassName = 'TORComboBox') then
         if gboxParam.Enabled then
              (ChildControl as TORComboBox).Color := clWindow
         else (ChildControl as TORComboBox).Color := clBtnFace;
    end;
end;

procedure SetComboBoxColor(cboxParam: TComboBox);
begin
  if cboxParam.Enabled then
       cboxParam.Color := clWindow
  else cboxParam.Color := clBtnFace;
end;

procedure PerformGridTitleSort(MyGrid: TDBGrid; MyField: TField; AscDesc: String);
var
  FGridToSort: TDBGrid;
  FSelectedField: TField;
  FSortOrder: String;
  nNumberOfSQLLines: Integer;
  sFieldName, sOriginTableName, sOriginFieldName, sTableAliasName, sOrderBy: String;

  // nested function...
  function ExtractOriginTableName(const sOrigin: String): String;
  var
    iPeriodPosition: Integer;
  begin
    // to get the table name we need to find the period separating it from the field name
    Result := '';
    iPeriodPosition := Pos('.', sOrigin);
    if iPeriodPosition > 0 then
       Result := Copy(sOrigin, 1, iPeriodPosition - 1);
  end;

  function ExtractOriginFieldName(const sOrigin: String): String;
  var
    iPeriodPosition: Integer;
  begin
    // to get the table name we need to find the period separating it from the field name
    Result := '';
    iPeriodPosition := Pos('.', sOrigin);
    if iPeriodPosition > 0 then
        Result := Copy(sOrigin, iPeriodPosition + 1, Length(sOrigin) - iPeriodPosition);
  end;

  function FindTableAliasName(const sOriginalTableName: String; const sSQLString: String): String;
  var
    iTableNamePosition, iAliasNameStartPos: Integer;
    sAliasName: String;
  begin
    Result := '';
    sAliasName := '';
    // ok, we need to find the table name surrounded by spaces or a comma on
    // the left side and a space on the right side.
    iTableNamePosition := Pos(' ' + sOriginalTableName + ' ', sSQLString);
    if iTableNamePosition = 0 then
       iTablenamePosition := Pos(',' + sOriginalTableName + ' ', sSQLString);
    if iTableNamePosition > 0 then
       begin
          // if we find it, then the alias has to have a comma, carriage return or no other character after it (end of string)
          // the first character after the table name must be space or a carriage return
          iAliasNameStartPos := iTableNamePosition + Length(sOriginalTableName) + 1;
          // find the first non-blank char
          while (sSQLString[iAliasNameStartPos] = ' ') and (iAliasNameStartPos < Length(sSQLString)) do
                 Inc(iAliasNameStartPos);
          while (iAliasNameStartPos < Length(sSQLString)) and not (sSQLString[iAliasNameStartPos] in [',', ' ', #13]) do
            begin
              sAliasName := sAliasName + sSQLString[iAliasNameStartPos];
              Inc(iAliasNameStartPos);
            end;
          Result := Trim(sAliasName);
       end;  // if iTableNamePosition > 0 then
  end;

begin  // PerformGridTitleSort
  FGridToSort := MyGrid;
  FSelectedField := MyField;
  FSortOrder := AscDesc;
  nNumberOfSQLLines := (FGridToSort.DataSource.DataSet as TQuery).SQL.count;
  sFieldName := FSelectedField.FieldName;
  sOriginTableName := ExtractOriginTableName(FSelectedField.Origin);
  sOriginFieldName := ExtractOriginFieldName(FSelectedField.Origin);
  if sOriginFieldName <> '' then sFieldName := sOriginFieldName;
  if sOriginTableName <> '' then
     begin
       sTableAliasName := FindTableAliasName(sOriginTableName,
	  		(FGridToSort.DataSource.DataSet as TQuery).SQL.Text);
       if sTableAliasName <> '' then
            sFieldName := sTableAliasName + '.' + sFieldName
       else sFieldName := sOriginTableName + '.' + sFieldName;
     end;
  sOrderBy := ' ORDER BY ' + sFieldName + ' ' + FSortOrder;
  if Pos('ORDER BY', UpperCase((FGridToSort.DataSource.DataSet as TQuery).SQL.Strings[nNumberOfSQLLines -1])) <> 0 then
       (FGridToSort.DataSource.DataSet as TQuery).SQL.Strings[nNumberOfSQLLines -1] := sOrderBy
  else (FGridToSort.DataSource.DataSet as TQuery).SQL.Add(sOrderBy);
  Screen.Cursor := crHourglass;
  (FGridToSort.DataSource.DataSet as TQuery).DisableControls;
  (FGridToSort.DataSource.DataSet as TQuery).Close;
  (FGridToSort.DataSource.DataSet as TQuery).Open;
  (FGridToSort.DataSource.DataSet as TQuery).EnableControls;
  FGridToSort.SelectedField := FSelectedField;
  Screen.Cursor := crDefault;
end;

function SetupPopupMenuItems(MyGrid: TDBGrid; MyPopupMenu: TPopupMenu; MyColumn: TColumn): TField;
var
  gGridToSort: TDBGrid;
  pPopupMenu: TPopupMenu;
  fSelectedField: TField;
  cColumn: TColumn;
  nPoint: TPoint;
  iMenuItemsIndex: Integer;

  // nested function...
  function RemoveTildes(Const sIncomingString: String): String;
  var
    sTempString: String;
    x: Integer;
  begin
    sTempString := '';
    for x := 1 to Length(sIncomingString) do
      if sIncomingString[x] <> '~' then
           sTempString := sTempString + sIncomingString[x]
      else sTempString := sTempString + ' ';
    Result := Trim(sTempString);
  end;

begin  // SetupPopupMenuItems
  gGridToSort := MyGrid;
  pPopupMenu := MyPopupMenu;
  cColumn := MyColumn;
  fSelectedField := gGridToSort.DataSource.DataSet.FieldByName(cColumn.FieldName);
  if gGridToSort.DataSource <> nil then
     begin
       if not (gGridToSort.DataSource.DataSet.IsEmpty) then
          begin
            GetCursorPos(nPoint);
            // indicate the name of the column to sort or find on
            for iMenuItemsIndex := 0 to pPopupMenu.Items.Count -1 do
            begin
              if pPopupMenu.Items[iMenuItemsIndex].Name = 'mniSortAscending' then
                 pPopupMenu.Items[iMenuItemsIndex].Caption :=
                       'Sort ' + RemoveTildes(fSelectedField.DisplayLabel) + ' Ascending'
              else
                if pPopupMenu.Items[iMenuItemsIndex].Name = 'mniSortDescending' then
                   pPopupMenu.Items[iMenuItemsIndex].Caption :=
                       'Sort ' + RemoveTildes(fSelectedField.DisplayLabel) + ' Descending';
            end;
            pPopupMenu.Popup(nPoint.x, nPoint.y);
          end;
     end;
  Result := fSelectedField;
end;

function CallServer(RPC: String; CallingParams: Array of String; MultList: TStrings): TStrings;
const
  //ParamTypeStrings: array[TParamType] of string = ('Literal', 'Reference', 'List', 'Undefined');
  //Updated with RPC Broker Patch 40 - tjl
  ParamTypeStrings: array[TParamType] of string = ('Literal', 'Reference', 'List', 'Global', 'Empty', 'Stream', 'Undefined');
var
  i, j: integer;
  ss, eMsg: string;
// zzzzzzandria 2012-04-16 =====================================================
{$IFDEF AANTEST}
  aStart,aStop: TDateTime;
  anEvent: TRPCEventItem;
{$ENDIF}
// zzzzzzandria 2012-04-16 =====================================================
begin
  Result := TStringList.Create;

  with frmECMain.BrokerEC do
    begin
      if Socket = 0 then
           InfoBox('No VistA server connection!', IB_WARN, mb_OK)
      else RemoteProcedure := RPC;

      for i := 0 to High(CallingParams) do
        if Copy(CallingParams[i], 1, 1) = '@' then
           begin
             ss := CallingParams[i];
             Param[i].Value := Copy(ss, 2, Length(ss) - 1);
             Param[i].PType := Reference;
           end
        else
           begin
             Param[i].Value := CallingParams[i];
             Param[i].PType := Literal;
           end;

      eMsg := 'RPC: ' + RPC;
      for i := 0 to Param.Count - 1 do
         eMsg := eMsg + #13#10 + 'Param[' + IntToStr(i) + ']=' +
                 ParamTypeStrings[Param[i].PType] + #9 + Param[i].Value;

      i := Param.Count; // In case we have to add a list to the end....

      if MultList <> nil then
        begin
          for j := 0 to MultList.Count - 1 do
            begin
              Param[i].Mult[IntToStr(j)] := MultList[j];
              eMsg := eMsg + #13#10 + 'List[' + intToStr(j) + ']='+#9 + MultList[j];
            end;
          Param[i].PType := List;
        end;
{$IFDEF AANTEST}
      anEvent := nil; // zzzzzzandria 2012-04-19 ===============================
      try
        aStart := Now; // zzzzzzandria 2012-04-16 ==============================
        anEvent := getTRPCBEventItem(frmEcMain.brokerEC); // ===================
        Call;
        aStop := Now; // zzzzzzandria 2012-04-19 ===============================
        anEvent.AppendResults(frmEcMain.brokerEC.Results,aStart,aStop); // =====
        addRPCEvent(anEvent);  // zzzzzzandria 2012-04-16 ======================
{$ELSE}
      try
        Call;
{$ENDIF}
        Result := Results;
      except
        on E: EBrokerError do
           begin
             InfoBox('Broker RPC Error:' + #13 + eMsg, IB_BRKERR, mb_OK);
{$IFDEF AANTEST}
             E.message := eMsg + #13#10 + E.message;

             if Assigned(anEvent) then // ======================================
               anEvent.AppendError(E.Message); // ==============================
             addRPCEvent(anEvent);  // zzzzzzandria 2012-04-16 =================
{$ENDIF}
           end;
      end;
    end;
end;

function FormatDateTimeForDisplay(DateParam: TDate; TimeParam, DisplayParam: String): String;
var
  DatePiece, TimePiece, TempTime: String;
begin
  DatePiece := FormatDateTime('dd MMM yyyy', DateParam);
  TempTime  := TimeParam;
  if TempTime = '' then TempTime := TimeToStr(Time);
  if Length(DisplayParam) < 6 then
       TimePiece := FormatDateTime(DisplayParam, StrToTime(TempTime))
  else
     begin
       TempTime  := TempTime + '000000';
       TimePiece := Copy(TempTime, 1, 2) + ':' + Copy(TempTime, 3, 2) + ':' + Copy(TempTime, 5, 2);
     end;
  Result := UpperCase(DatePiece) + ' @' + TimePiece;
end;

//acs - gets the current DATE only (not the TIME)
function SetupDate(TextParam: String): TDate;
var
  ErrorDate: TDate;
  x: Integer;
  EditString, TempString, sYear, sMonth, sDay: String;
  Temp, Tempdd, sYearTime : String;
begin
  EditString := TextParam;
  TempString := '';
  // Remove all spaces
  for x := 1 to Length(EditString) do
     if EditString[x] <> ' ' then
        TempString := TempString + EditString[x];

  ErrorDate := EncodeDate(1900, 10, 10);
  Result := ErrorDate;

  if Length(TempString) = 0 then
     InfoBox('The Date cannot be empty.', IB_REQDATA, mb_OK)
  else if Pos('+', TempString) > 0 then
     InfoBox('The Date cannot be in the future.', IB_DATAERR, mb_OK)
  else
     begin
       frmEcMain.brokerEC.RemoteProcedure := 'EC GETDATE';
       frmEcMain.brokerEC.Param[0].Value := TempString;
       frmEcMain.brokerEC.Param[0].PType := literal;
       try
//         frmEcMain.brokerEC.Call;
         frmECMain.RPCBCall; // zzzzzzandria 2012-04-19 RPC Logging ============
       except
         On EBrokerError do
           // ShowMessage replaced by InfoBox - Patch 84 - tjl
           InfoBox('Broker Error at EC GETDATE.', IB_BRKERR, mb_OK);
       end;
       if Piece(frmEcMain.brokerEC.Results[0], '^', 1) = '0' then
          // ShowMessage replaced by InfoBox - Patch 84 - tjl
          InfoBox('Date cannot be in the future, or invalid date format.' + #13 +
                  'Use a valid FileMan format:' + #13#13 + 'dd MMM yyyy' + #13 +
                  'dd/mm/yy' + #13 + 'dd/mm/yyyy' + #13 + 'T, T-1, etc.', IB_DATAERR, mb_OK)
       else
         begin
           Temp := piece(frmEcMain.brokerEC.Results[0],'^',2);
           sMonth := SetMonthValue(piece(Temp,' ',1));
           Tempdd := piece(Temp,' ',2);
           sDay := piece(Tempdd,',',1);
           sYearTime := piece(Temp,' ',3);
           sYear := piece(sYearTime,'@',1);
           try
             Result := EncodeDate(StrToInt(sYear), StrToInt(sMonth), StrToInt(sDay));
           except
             on E: EConvertError do
                // ShowMessage replaced by InfoBox - Patch 84 - tjl
                InfoBox(E.ClassName + #13#10 + E.Message + #13#10 + #13#10 +
                        'Please enter a date in DD MMM YYYY format -' + #13#10 +
                        'for example:  17 MAR 2001', IB_DATAERR, mb_OK);
           end;
         end;
     end;
end;

//acs - added function to get the date and time
function GetDateTime(TextParam: String): String;
var
  EditString, sYear, sMonthNum, sDay, sTime: String;
  Temp, Tempdd, sYearTime, sCurrentDateTime : String;
begin
  EditString := Trim(TextParam);

  if Length(EditString) = 0 then
    begin
     InfoBox('The Procedure Date and Time field cannot be empty.', IB_REQDATA, mb_OK);
     Result := '0';
    end
  else if Pos('+', EditString) > 0 then
     begin
     InfoBox('The Procedure Date and Time value cannot be in the future.', IB_DATAERR, mb_OK);
     Result := '0';
     end
  else
     begin
//{$IFDEF AANTEST} // ICD-10. zzzzzzandria disabling validation for testing future dates
//        if TextParam = 'N' then
//          begin
//            sCurrentDateTime :=
//              FormatDateTime('mm/dd/yyyy^hh:nn',Now);
//          end
//        else
//          begin
//            sMonthNum := SetMonthValue(piece(TextParam,' ',1));
//            sDay := piece(piece(TextParam,' ',2),',',1);
//            sYear := piece(piece(TextParam,',',2),'@',1);
//            sTime := piece(TextParam,'@',2);
//            sCurrentDateTime := sMonthNum + '/' + sDay + '/' + sYear + '^' + sTime;
//          end;
////        ShowMessage('Day: '+sDay+#13#10+'Month: ' + sMonthNum + #13#10 +
////          'Year: ' + sYear + #13#10+ 'Time: '+ sTime + #13#10 + sCurrentDateTime);
//{$ELSE}
        sCurrentDateTime := '0';
        frmEcMain.brokerEC.RemoteProcedure := 'EC GETDATE';
        frmEcMain.brokerEC.Param[0].Value := EditString + '^R'; // R => time is required
        frmEcMain.brokerEC.Param[0].PType := literal;
        try
//          frmEcMain.brokerEC.Call;
          frmECMain.RPCBCall; // zzzzzzandria 2012-04-19 RPC Logging ===========
        except
          On EBrokerError do
             // ShowMessage replaced by InfoBox - Patch 84 - tjl
             InfoBox('Broker Error at EC GETDATE.', IB_BRKERR, mb_OK);
        end;
        // ShowMessage replaced by InfoBox - Patch 84 - tjl
        if Piece(frmEcMain.brokerEC.Results[0], '^', 1) = '0' then
           // added display of input and restored @hhmm in message
           InfoBox('Date ' + EditString + ' cannot be in the future, or invalid date and time format.' + #13 +
                'Use a valid FileMan format:' + #13#13 +
                'dd MMM yyyy @hhmm' + #13 +
                'mm/dd/yy @hhmm' + #13 +
                'mm/dd/yyyy @hhmm' + #13 +
                'T@hhmm, T-1@hhmm, etc.', IB_DATAERR, mb_OK)
           // InfoBox('Date cannot be in the future, or invalid date format.' + #13 +
           //     'Use a valid FileMan format:' + #13#13 + 'dd MMM yyyy' + #13 +
           //      'dd/mm/yy' + #13 + 'dd/mm/yyyy' + #13 + 'T, T-1, etc.', IB_DATAERR, mb_OK)
//          ShowMessage('Date cannot be in the future, or Invalid date and time format.' + #13 +
//          'Use a valid FileMan format:' + #13 + #13 +
//          'dd MMM yyyy @hhmm' + #13 +
//          'dd/mm/yy @hhmm' + #13 +
//          'dd/mm/yyyy @hhmm' + #13 +
//          'N, N-1, N-1H, etc.')
        else
          begin
            Temp := piece(frmEcMain.brokerEC.Results[0],'^',2);
            sMonthNum := SetMonthValue(piece(Temp,' ',1));
            Tempdd := piece(Temp,' ',2);
            sDay := piece(Tempdd,',',1);
            sYearTime := piece(Temp,' ',3);
            sYear := trim(piece(sYearTime,'@',1));
            sTime := trim(piece(sYearTime,'@',2));
            //put the FileMan date/time into the correct format: 'mm/dd/yyyy hhmm'
            sCurrentDateTime := sMonthNum + '/' + sDay + '/' + sYear + '^' + sTime;
          end;
//{$ENDIF} // ICD-10. zzzzzzandria disabling validation for testing future dates
        try
          Result := sCurrentDateTime;
        except
          //Modified for Patch 57.  Removed colon betweeen hours and minutes.
          on E: EConvertError do
                // ShowMessage replaced by InfoBox - Patch 84 - tjl
                InfoBox(E.ClassName + #13#10 + E.Message + #13#10 + #13#10 +
                        'Please enter a date in DD MMM YYYY @HHMM format -' + #13#10 +
                        'for example:  17 MAR 2001 @1400', IB_DATAERR, mb_OK);
        end;
     end;
end;


function SetMonthValue(MonthParam: String): String;
begin
  if MonthParam = 'JAN' then Result := '1';
  if MonthParam = 'FEB' then Result := '2';
  if MonthParam = 'MAR' then Result := '3';
  if MonthParam = 'APR' then Result := '4';
  if MonthParam = 'MAY' then Result := '5';
  if MonthParam = 'JUN' then Result := '6';
  if MonthParam = 'JUL' then Result := '7';
  if MonthParam = 'AUG' then Result := '8';
  if MonthParam = 'SEP' then Result := '9';
  if MonthParam = 'OCT' then Result := '10';
  if MonthParam = 'NOV' then Result := '11';
  if MonthParam = 'DEC' then Result := '12';
end;

function ConvertTwoDigitYear(YearParam: Integer): String;
var
  CurrentYear, CurrentCentury: Integer;
begin
  CurrentYear := StrToInt(FormatDateTime('yyyy', Date));
  CurrentCentury := CurrentYear div 100;
  if (CurrentYear - (CurrentCentury * 100) - YearParam) >= 0 then
       Result := IntToStr((CurrentCentury * 100) + YearParam)
  else Result := IntToStr(((CurrentCentury - 1) * 100) + YearParam);
end;

function DateToMDate(DateParam: String): String;
var
  sDay, sMonth, sYear: String;
begin
  sDay := Copy(DateParam, 6, 2);
  sMonth := Copy(DateParam, 4, 2);
  sYear := IntToStr(1700 + StrToInt(Copy(DateParam, 1, 3)));
  Result := DateToStr(EncodeDate(StrToInt(sYear), StrToInt(sMonth), StrToInt(sDay)));
end;


function WindowsDateToFMDate(WinDate: TDate): Double; // zzzzzzandria 2012-04-16
var
  Year, Month, Day: Word;
begin
  DecodeDate(WinDate, Year, Month, Day);
  Result := ((Year - 1700) * 10000) + (Month * 100) + Day;
end;

function SetMdateOfProcedure(TextParam: String): String;
var
  TempDateTime, TempYear, TempMonth, TempDay: String;
begin
  if TextParam <> '' then    // modified for patch 59 NOIS ATG-1003-31818 by VMP
     begin
       TempDateTime := Trim(Piece(TextParam, '@', 1));
       TempYear     := Trim(Piece(TempDateTime, ' ', 3));
       TempYear     := IntToStr(StrToInt(TempYear) - 1700);
       TempMonth    := SetMonthValue(UpperCase(Trim(Piece(TempDateTime, ' ', 2))));
       if Length(TempMonth) = 1 then TempMonth := '0' + TempMonth;
       TempDay      := Trim(Piece(TempDateTime, ' ', 1));
       Result       := TempYear + TempMonth + TempDay;
     end;
end;

function SubSetOfRecords(const SearchParams: String): TStrings;
begin
  CallServer('EC GETLIST', SearchParams, nil);
  Result := frmECMain.brokerEC.Results;
end;

function GetGenericIEN(IndexParam, ValueParam, FileNoParam, FileNameParam: String): String;
var
  RpcParam: String;
  IenReturned: TStrings;
begin
  try
    IenReturned := TStringList.Create;
    RpcParam := FileNoParam + '^' + ValueParam;
    IenReturned.Assign(CallServer('EC GETIEN', RpcParam, nil));
    if IenReturned.Count > 0 then
       Result := Piece(IenReturned[0], '^', 1);
    IenReturned.Free;
  except
    InfoBox('Error retrieving the ' + FileNameParam + ' IEN.', IB_BRKERR, mb_OK);
  end;
end;


function CompareVersion(const A, B: String): Integer;
var
  NumA, NumB: Integer;
begin
  NumA := (StrToInt(Piece(A, '.', 1)) * 16777216) +
          (StrToInt(Piece(A, '.', 2)) * 65536) +
          (StrToInt(Piece(A, '.', 3)) * 256) +
           StrToInt(Piece(A, '.', 4));
  NumB := (StrToInt(Piece(B, '.', 1)) * 16777216) +
          (StrToInt(Piece(B, '.', 2)) * 65536) +
          (StrToInt(Piece(B, '.', 3)) * 256) +
           StrToInt(Piece(B, '.', 4));
  Result := NumA - NumB;
end;

function InfoBox(const Text, Caption: string; Flags: Word): Integer;
{ wrap the messagebox object in case we want to modify it later }
begin
  Result := Application.MessageBox(PChar(Text), PChar(Caption), Flags or MB_TOPMOST);
end;

// Patch 84 - tjl - indicates if JAWS or Window-Eyes is running
function ScreenReaderActive: Boolean;
var
  bReaderRunning, bReaderStateOn: BOOL;
begin
  bReaderRunning := SystemParametersInfo(SPI_GETSCREENREADER, 0, @bReaderStateOn, 0);
  Result := (bReaderRunning and bReaderStateOn);
end;


{** Timeout Functions **}

function TimeoutKeyHook(Code: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT;
{ this is called for every keyboard event that occurs while running Event Capture }
begin
  if lParam shr 31 = 1 then tmrTimeout.ResetTimeout;  // on KeyUp only
  Result := CallNextHookEx(tmrTimeout.uTimeoutKeyHandle, Code, wParam, lParam);
end;

function TimeoutMouseHook(Code: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT;
{ this is called for every mouse event that occurs while running Event Capture }
begin
  if (Code >= 0) and (wParam > WM_MOUSEFIRST) and (wParam <= WM_MOUSELAST)
     then tmrTimeout.ResetTimeout;                    // all click events
  Result := CallNextHookEx(tmrTimeout.uTimeoutMouseHandle, Code, wParam, lParam);
end;

procedure InitTimeOut(AUserCondition: TECSTimeoutCondition;
                      AUserAction: TECSTimeoutAction);
begin
  if not Assigned(tmrTimeout) then
     begin
       tmrTimeout := TECSTimeoutTimer.Create(Application);
       with tmrTimeout do
         begin
           OnTimer := EcsTimeout;
           FUserCondition := AUserCondition;
           FUserAction := AUserAction;
           uTimeoutInterval    := 120000;  // initially 2 minutes, will get FiTimeoutInterval after signon
           uTimeoutKeyHandle   := SetWindowsHookEx(WH_KEYBOARD, TimeoutKeyHook,   0, GetCurrentThreadID);
           uTimeoutMouseHandle := SetWindowsHookEx(WH_MOUSE,    TimeoutMouseHook, 0, GetCurrentThreadID);
           FHooked  := TRUE;
           Interval := uTimeoutInterval;
           Enabled  := TRUE;
         end;
     end;
end;

procedure UpdateTimeOutInterval(NewTime: Cardinal);
begin
  if Assigned(tmrTimeout) then
     with tmrTimeout do
       begin
         uTimeoutInterval := NewTime;
         Interval := uTimeoutInterval;
         Enabled  := TRUE;
       end;
end;

function TimedOut: Boolean;
begin
  Result := FTimedOut;
end;

procedure ShutDownTimeOut;
begin
  if Assigned(tmrTimeout) then
     begin
       with tmrTimeout do
         begin
           Enabled := FALSE;
           if FHooked then
             begin
               UnhookWindowsHookEx(uTimeoutKeyHandle);
               UnhookWindowsHookEx(uTimeoutMouseHandle);
               FHooked := FALSE;
             end;
         end;
       tmrTimeout.Free;
       tmrTimeout := nil;
     end;
end;

{ TECSTimeoutTime }

procedure TEcsTimeoutTimer.ResetTimeout;
{ this restarts the timer whenever there is a keyboard or mouse event }
begin
  Enabled  := FALSE;
  Interval := uTimeoutInterval;
  Enabled  := TRUE;
end;

procedure TEcsTimeoutTimer.ECSTimeout(Sender: TObject);
{ when the timer expires, the application is closed after warning the user }
begin
  Enabled := FALSE;
  FTimedOut := AllowTimeout;                     // called from UfrmEcTimeout
  if FTimedOut then
     if Assigned(FUserAction) then FUserAction;  // calls UfrmEcMain.Close
  Enabled := not FTimedOut;
end;

// if match error code in first piece of results, display messages
// use StartPiece to ignore unwanted pieces such as an IEN in the error messages.
// rpk 6/27/2008
procedure ShowRPCErrors(ErrorCode: String; Prompt: String; StartPiece: Integer);
const
     up = '^';
var
   Msg, resstr, pstr : String;
   i, j, ilen, icnt, pcnt : Integer;
begin
      if Piece(frmEcMain.brokerEC.Results[0], up, 1) = ErrorCode then begin
         Msg := Prompt + ' ';
         icnt := frmEcMain.brokerEC.Results.Count;
         resstr := '';
         for i := 0 to icnt - 1 do begin
             pstr := frmEcMain.brokerEC.Results[i];
             pcnt := DelimCount(pstr, up) + 1;
             resstr := Pieces(pstr, up, StartPiece, pcnt);
             ilen := Length(resstr);
             for j := 1 to ilen - 1 do
                 if resstr[j] = up then
                    resstr[j] := ',';
             Msg := Msg + resstr + #13#10;
             //  + Piece(frmEcMain.brokerEC.Results[i], '^', 3) + #13#10;
         end;
         InfoBox(Msg, IB_NOTFILED, mb_OK);
      end;
end;  // ShowRPCErrors

{==============================================================================
  This is a simple procedure to display an information message concerning captionlistboxes that
  cannot be read by screen reader software. More specifically, the headers for caption list boxes
  are not read/recognized.

  Parameters:
    HdrCntl: is the THeaderControl to pull column headings from
    Kind: is an integer variable indicating what type of message values
      1 - Message displayed with the listbox is first entered
      2 - Message displayed when a user presses F1 within the box

  Modification History:
    EC112 EC FY12 -  Procedure created -CVW
 ==============================================================================}
procedure CaptionListbox508msg(HdrCntl: Theadercontrol; kind: integer ) ;
var
  indx: integer ;
  msgstr : string ;
begin
  case kind of
  1:  begin
        msgstr := 'This is a Caption List Box that displays data in a column format that is not readable by ' ;
        msgstr := msgstr + 'screen reader technology. The columns for this Caption List box are' ;
        for indx := 0 to HdrCntl.Sections.Count - 1 do
        begin
          msgstr := msgstr + ': ' + HdrCntl.Sections[indx].Text ;
        end ;
        msgstr := msgstr + '. While in this list box, if you press F1, you will be given the column headings again. A permanent fix is forthcoming. ' ;
        InfoBox(msgstr, IB_INFO, mb_OK);
      end;
  2:  begin
        msgstr := 'The columns for this Caption List box are' ;
        for indx := 0 to HdrCntl.Sections.Count - 1 do
        begin
          msgstr := msgstr + ': ' + HdrCntl.Sections[indx].Text ;
        end ;
        InfoBox(msgstr, IB_INFO, mb_OK);
      end
  end;
end; // CaptionListbox508msg

//Added function for EC FY 13 - CVW simple function to test if string is a number
//updated to include decimal and minus
function IsStrANumber(const S: string): Boolean;
var
P: PChar;
begin
  P := PChar(S);
  Result := False;
  while P^ <> #0 do
  begin
    if not (P^ in ['0'..'9','.','-']) then Exit;
    Inc(P);
  end;
  Result := True;
end ;

{==============================================================================
  This is a procedure to dynamically place controls within a group box.
  It was created out of the need to force screen reader software to announce
  labels that were not being read correctly in Windows 7.

  Parameters:
    TargetControl: TWinControl
      This is the TWinControl to be put into a group box
    GBCaption: string
      This is the caption for the new group box
    ParentForm: TForm
      This is the parent form for the new group box

  Modification History:
    EC FY14 -  Procedure created 03/13 -MER CVW
 ==============================================================================}
procedure PutControlInGroupBox(TargetControl: TWinControl; GBCaption: string; ParentForm: TForm);
var
  NewGB:  TGroupBox;
begin
  // One must pass the parent form as the owner so that the dynamically created
  // component is destroyed when the form is destroyed
  NewGB := TGroupBox.Create(ParentForm);
  // Set the parent to the target control's parent
  newGB.Parent := TargetControl.Parent;
  NewGB.Caption := GBCaption;
  // Position the new groupBox
  if (TargetControl.Top - 12) < 0 then
    NewGB.Top := 0
  else
    NewGB.Top := TargetControl.Top - 12;
  if (TargetControl.Left - 3) < 0 then
    NewGB.Left := 0
  else
    NewGB.Left := TargetControl.Left - 3;
  NewGB.Width := TargetControl.Width + 6;
  NewGB.Height := TargetControl.Height +16;
  NewGB.TabOrder := TargetControl.TabOrder;
  // Assign parent and location for the existing control
  TargetControl.Parent := NewGB;
  TargetControl.Top := 12;
  TargetControl.Left := 3;
  TargetControl.TabOrder := 0;
end;


end.
